<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"    
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" >

<!-- (c) 2011, Trimble Navigation Limited. All rights reserved.                                -->
<!-- Permission is hereby granted to use, copy, modify, or distribute this style sheet for any -->
<!-- purpose and without fee, provided that the above copyright notice appears in all copies   -->
<!-- and that both the copyright notice and the limited warranty and restricted rights notice  -->
<!-- below appear in all supporting documentation.                                             -->

<!-- TRIMBLE NAVIGATION LIMITED PROVIDES THIS STYLE SHEET "AS IS" AND WITH ALL FAULTS.         -->
<!-- TRIMBLE NAVIGATION LIMITED SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY -->
<!-- OR FITNESS FOR A PARTICULAR USE. TRIMBLE NAVIGATION LIMITED DOES NOT WARRANT THAT THE     -->
<!-- OPERATION OF THIS STYLE SHEET WILL BE UNINTERRUPTED OR ERROR FREE.                        -->

<xsl:output method="text" omit-xml-declaration="yes" encoding="ISO-8859-1"/>

<!-- Set the numeric display details i.e. decimal point, thousands separator etc -->
<xsl:variable name="DecPt" select="'.'"/>    <!-- Change as appropriate for US/European -->
<xsl:variable name="GroupSep" select="','"/> <!-- Change as appropriate for US/European -->
<!-- Also change decimal-separator & grouping-separator in decimal-format below 
     as appropriate for US/European output -->
<xsl:decimal-format name="Standard" 
                    decimal-separator="."
                    grouping-separator=","
                    infinity="Infinity"
                    minus-sign="-"
                    NaN="?"
                    percent="%"
                    per-mille="&#2030;"
                    zero-digit="0" 
                    digit="#" 
                    pattern-separator=";" />

<xsl:variable name="DecPl0" select="'#0'"/>
<xsl:variable name="DecPl1" select="concat('#0', $DecPt, '0')"/>
<xsl:variable name="DecPl2" select="concat('#0', $DecPt, '00')"/>
<xsl:variable name="DecPl3" select="concat('#0', $DecPt, '000')"/>
<xsl:variable name="DecPl4" select="concat('#0', $DecPt, '0000')"/>
<xsl:variable name="DecPl5" select="concat('#0', $DecPt, '00000')"/>
<xsl:variable name="DecPl6" select="concat('#0', $DecPt, '000000')"/>
<xsl:variable name="DecPl8" select="concat('#0', $DecPt, '00000000')"/>

<xsl:variable name="fileExt" select="'txt'"/>

<!-- User variable definitions - Appropriate fields are displayed on the       -->
<!-- Survey Controller screen to allow the user to enter specific values       -->
<!-- which can then be used within the style sheet definition to control the   -->
<!-- output data.                                                              -->
<!--                                                                           -->
<!-- All user variables must be identified by a variable element definition    -->
<!-- named starting with 'userField' (case sensitive) followed by one or more  -->
<!-- characters uniquely identifying the user variable definition.             -->
<!--                                                                           -->
<!-- The text within the 'select' field for the user variable description      -->
<!-- references the actual user variable and uses the '|' character to         -->
<!-- separate the definition details into separate fields as follows:          -->
<!-- For all user variables the first field must be the name of the user       -->
<!-- variable itself (this is case sensitive) and the second field is the      -->
<!-- prompt that will appear on the Survey Controller screen.                  -->
<!-- The third field defines the variable type - there are four possible       -->
<!-- variable types: Double, Integer, String and StringMenu.  These variable   -->
<!-- type references are not case sensitive.                                   -->
<!-- The fields that follow the variable type change according to the type of  -->
<!-- variable as follow:                                                       -->
<!-- Double and Integer: Fourth field = optional minimum value                 -->
<!--                     Fifth field = optional maximum value                  -->
<!--   These minimum and maximum values are used by the Survey Controller for  -->
<!--   entry validation.                                                       -->
<!-- String: No further fields are needed or used.                             -->
<!-- StringMenu: Fourth field = number of menu items                           -->
<!--             Remaining fields are the actual menu items - the number of    -->
<!--             items provided must equal the specified number of menu items. -->
<!--                                                                           -->
<!-- The style sheet must also define the variable itself, named according to  -->
<!-- the definition.  The value within the 'select' field will be displayed in -->
<!-- the Survey Controller as the default value for the item.                  -->

<!-- Need to use the select attribute for userField definitions and also include the -->
<!-- contents in single quotes so that the user information can be picked up by the  -->
<!-- Survey Controller and ASCII File Generator.  This means it is not possible to   -->
<!-- have text that includes any apostrophes (single quotes) in it.                  -->
<xsl:variable name="userField1" select="'sortType|Point name sort|stringMenu|2|Numerical|Alphabetical'"/>
<xsl:variable name="sortType" select="'Alphabetical'"/>

<xsl:key name="obsID-search" match="/JOBFile/FieldBook/PointRecord" use="@ID"/>

<!-- **************************************************************** -->
<!-- Set global variables from the Environment section of JobXML file -->
<!-- **************************************************************** -->
<xsl:variable name="DistUnit"   select="/JOBFile/Environment/DisplaySettings/DistanceUnits" />
<xsl:variable name="AngleUnit"  select="/JOBFile/Environment/DisplaySettings/AngleUnits" />
<xsl:variable name="CoordOrder" select="/JOBFile/Environment/DisplaySettings/CoordinateOrder" />
<xsl:variable name="TempUnit"   select="/JOBFile/Environment/DisplaySettings/TemperatureUnits" />
<xsl:variable name="PressUnit"  select="/JOBFile/Environment/DisplaySettings/PressureUnits" />

<!-- Setup conversion factor for coordinate and distance values -->
<!-- Dist/coord values in JobXML file are always in metres -->
<xsl:variable name="DistConvFactor">
  <xsl:choose>
    <xsl:when test="$DistUnit='Metres'">1.0</xsl:when>
    <xsl:when test="$DistUnit='InternationalFeet'">3.280839895</xsl:when>
    <xsl:when test="$DistUnit='USSurveyFeet'">3.2808333333357</xsl:when>
    <xsl:otherwise>1.0</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup conversion factor for angular values -->
<!-- Angular values in JobXML file are always in decimal degrees -->
<xsl:variable name="AngleConvFactor">
  <xsl:choose>
    <xsl:when test="$AngleUnit='DMSDegrees'">1.0</xsl:when>
    <xsl:when test="$AngleUnit='Gons'">1.111111111111</xsl:when>
    <xsl:when test="$AngleUnit='Mils'">17.77777777777</xsl:when>
    <xsl:otherwise>1.0</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup boolean variable for coordinate order -->
<xsl:variable name="NECoords">
  <xsl:choose>
    <xsl:when test="$CoordOrder='North-East-Elevation'">true</xsl:when>
    <xsl:when test="$CoordOrder='X-Y-Z'">true</xsl:when>
    <xsl:otherwise>false</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup conversion factor for pressure values -->
<!-- Pressure values in JobXML file are always in millibars (hPa) -->
<xsl:variable name="PressConvFactor">
  <xsl:choose>
    <xsl:when test="$PressUnit='MilliBar'">1.0</xsl:when>
    <xsl:when test="$PressUnit='InchHg'">0.029529921</xsl:when>
    <xsl:when test="$PressUnit='mmHg'">0.75006</xsl:when>
    <xsl:otherwise>1.0</xsl:otherwise>
  </xsl:choose>
</xsl:variable>


<!-- **************************************************************** -->
<!-- ************************** Main Loop *************************** -->
<!-- **************************************************************** -->
<xsl:template match="/" >

  <xsl:choose>
    <xsl:when test="$NECoords = 'true'">
      <xsl:value-of select="'Point;North;East;Elev;HzPrec;VtPrec;PDOP;Sats;NumPosUsed'"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="'Point;East;North;Elev;HzPrec;VtPrec;PDOP;Sats;NumPosUsed'"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:call-template name="NewLine"/>

  <!-- Select Reductions node to process -->
  <xsl:apply-templates select="JOBFile/Reductions" />

</xsl:template>


<!-- **************************************************************** -->
<!-- ***************** Reductions Node Processing ******************* -->
<!-- **************************************************************** -->
<xsl:template match="Reductions">
  <xsl:choose>
    <xsl:when test="$sortType = 'Numerical'">
      <xsl:apply-templates select="Point">
        <xsl:sort data-type="number" order="ascending" select="Name"/>
      </xsl:apply-templates>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="Point">
        <xsl:sort data-type="text" order="ascending" select="Name"/>
      </xsl:apply-templates>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- **************** Grid Point Details Output ********************* -->
<!-- **************************************************************** -->
<xsl:template match="Point">
  <xsl:variable name="PtStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="12"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="Name" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="NthStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="13"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number(Grid/North * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="EastStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="13"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number(Grid/East * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="ElevStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="9"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number(Grid/Elevation * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="CodeStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="20"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="Code" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="HorizPrec">
    <xsl:for-each select="key('obsID-search', ID)">
      <xsl:value-of select="Precision/Horizontal"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="hzPrec">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number($HorizPrec * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="VertPrec">
    <xsl:for-each select="key('obsID-search', ID)">
      <xsl:value-of select="Precision/Vertical"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="vtPrec">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number($VertPrec * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="tempPDOP">
    <xsl:for-each select="key('obsID-search', ID)">
      <xsl:value-of select="QualityControl1/PDOP"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="PDOP">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="5"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number($tempPDOP, $DecPl1, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="tempNbrSat">
    <xsl:for-each select="key('obsID-search', ID)">
      <xsl:value-of select="QualityControl1/NumberOfSatellites"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="NbrSat">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="5"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number($tempNbrSat, $DecPl0, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="tempNbrPosUsed">
    <xsl:for-each select="key('obsID-search', ID)">
      <xsl:value-of select="QualityControl1/NumberOfPositionsUsed"/>
    </xsl:for-each>
  </xsl:variable>
  
  <xsl:variable name="NbrPosUsed">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="5"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number($tempNbrPosUsed, $DecPl0, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>  
  
  <xsl:choose>
    <xsl:when test="$NECoords = 'true'">
      <xsl:value-of select="concat($PtStr, ';', $NthStr, ';', $EastStr, ';', $ElevStr, ';', $hzPrec, ';', $vtPrec, ';', $PDOP, ';', $NbrSat, ';', $NbrPosUsed)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="concat($PtStr, ';', $EastStr, ';', $NthStr, ';', $ElevStr, ';', $hzPrec, ';', $vtPrec, ';', $PDOP, ';', $NbrSat, ';', $NbrPosUsed)"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:call-template name="NewLine"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the right with spaces ************** -->
<!-- **************************************************************** -->
<xsl:template name="PadRight">
  <xsl:param name="stringWidth"/>
  <xsl:param name="theString"/>
  <xsl:choose>
    <xsl:when test="$stringWidth = '0'">
      <xsl:value-of select="normalize-space($theString)"/> <!-- Function return value -->
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="normalize-space($theString)"/> <!-- Function return value -->    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the left with spaces *************** -->
<!-- **************************************************************** -->
<xsl:template name="PadLeft">
  <xsl:param name="stringWidth"/>
  <xsl:param name="theString"/>
  <xsl:choose>
    <xsl:when test="$stringWidth = '0'">
      <xsl:value-of select="normalize-space($theString)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="normalize-space($theString)"/> <!-- Function return value -->    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** New Line Output ************************* -->
<!-- **************************************************************** -->
<xsl:template name="NewLine">
  <xsl:text>&#10;</xsl:text>
</xsl:template>


</xsl:stylesheet>
